package xyz.lychee.lagfixer.nms.v1_21_R2;

import it.unimi.dsi.fastutil.objects.ObjectIterator;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.level.EntityPlayer;
import net.minecraft.server.level.PlayerChunkMap;
import net.minecraft.server.level.WorldServer;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityCreature;
import net.minecraft.world.entity.ai.targeting.PathfinderTargetCondition;
import net.minecraft.world.entity.decoration.EntityArmorStand;
import org.bukkit.World;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.craftbukkit.v1_21_R2.CraftWorld;
import org.bukkit.craftbukkit.v1_21_R2.entity.CraftCreature;
import xyz.lychee.lagfixer.modules.CustomTrackerModule;
import xyz.lychee.lagfixer.utils.ReflectionUtils;

public class CustomTracker extends CustomTrackerModule.NMS {
   private final boolean disableTicking;
   private final double range;
   private final PathfinderTargetCondition target;
   private Method addEntityMethod;
   private Field entityField;

   public CustomTracker(CustomTrackerModule module) {
      super(module);
      ConfigurationSection section = this.getModule().getSection();
      this.disableTicking = section.getBoolean("disable_ticking");
      this.range = section.getDouble("range");
      this.target = PathfinderTargetCondition.b().a(this.range).d();

      try {
         this.addEntityMethod = ReflectionUtils.getPrivateMethod(PlayerChunkMap.class, "a", new Class[]{Entity.class});
         this.entityField = ReflectionUtils.getPrivateField(PlayerChunkMap.EntityTracker.class, Entity.class);
      } catch (SecurityException | IllegalArgumentException | NoSuchMethodException var4) {
         Exception ex = var4;
         this.getModule().getPlugin().printError(ex);
      }

   }

   public void setAware(org.bukkit.entity.Entity ent, boolean bl) {
      if (ent instanceof CraftCreature) {
         ((CraftCreature)ent).getHandle().aware = bl;
      }

   }

   public void setTicking(Entity entity, boolean bl) {
      if (entity != null && entity.valid && !this.disableTicking) {
         entity.activatedTick = bl ? (long)MinecraftServer.currentTick : Long.MIN_VALUE;
         if (entity instanceof EntityCreature) {
            ((EntityCreature)entity).aware = bl;
         }

      }
   }

   public void checkWorld(World world) {
      WorldServer ws = ((CraftWorld)world).getHandle();
      PlayerChunkMap map = ws.m().a;
      Iterator var4 = ws.y().iterator();

      while(var4.hasNext()) {
         EntityPlayer player = (EntityPlayer)var4.next();
         ws.a(EntityCreature.class, this.target, player, player.cR().g(this.range)).stream().filter((ent) -> {
            return !map.K.containsKey(ent.ar()) && ent.valid;
         }).forEach((ent) -> {
            try {
               this.addEntityMethod.invoke(map, ent);
            } catch (InvocationTargetException | IllegalAccessException var4) {
               ReflectiveOperationException ex = var4;
               this.getModule().getPlugin().printError(ex);
            }

            this.setTicking(ent, true);
            this.getModule().getPlugin().getLogger().info("Tracked entity " + ent.ar());
         });
      }

   }

   public void untrackProcess(World world) {
      PlayerChunkMap map = ((CraftWorld)world).getHandle().m().a;
      ObjectIterator var3 = map.K.values().iterator();

      while(var3.hasNext()) {
         PlayerChunkMap.EntityTracker tracked = (PlayerChunkMap.EntityTracker)var3.next();

         Entity ent;
         try {
            ent = (Entity)this.entityField.get(tracked);
         } catch (IllegalAccessException | IllegalArgumentException var8) {
            Exception ex = var8;
            this.getModule().getPlugin().printError(ex);
            continue;
         }

         if (!(ent instanceof EntityPlayer) && !ent.getClass().getName().contains("boss") && (!(ent instanceof EntityArmorStand) || ent.an() == null)) {
            boolean remove = tracked.f.isEmpty() || tracked.f.size() == 1 && tracked.f.stream().anyMatch((pc) -> {
               return pc.o().f.isDisconnected();
            });
            if (remove) {
               PlayerChunkMap.EntityTracker removed = (PlayerChunkMap.EntityTracker)map.K.remove(ent.ar());
               if (removed != null) {
                  removed.a();
               }

               this.setTicking(ent, false);
               this.getModule().getPlugin().getLogger().info("Untracked entity " + ent.ar());
            }
         } else {
            this.getModule().getPlugin().getLogger().info("Skipped untracking");
         }
      }

   }
}
